from datetime import datetime, timezone


def get_age_in_hrs_from_unix(unix_timestamp):
    try:
        if unix_timestamp:
            current_time = datetime.now()
            target_time = datetime.fromtimestamp(unix_timestamp)

            time_difference = current_time - target_time
            hours_from_now = time_difference.total_seconds() / 3600
            return hours_from_now
        else:
            return None
    except Exception as e:
        print(f"got error{e}")
        return None


def time_ago_from_timestamp_string(timestamp):
    try:
        now = datetime.now(timezone.utc)

        if isinstance(timestamp, str):
            try:
                dt = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%S.%fZ").replace(tzinfo=timezone.utc)
            except ValueError:
                dt = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
        else:
            dt = datetime.fromtimestamp(timestamp, tz=timezone.utc)

        # Calculate the difference between the current time and the given timestamp
        diff = now - dt

        # Convert the difference to minutes, hours, or days
        minutes = diff.total_seconds() // 60
        hours = diff.total_seconds() // 3600
        days = diff.days

        # Determine the appropriate time unit
        if days > 0:
            return f"{int(days)}d"
        elif hours > 0:
            return f"{int(hours)}h"
        elif minutes > 0:
            return f"{int(minutes)}m"
        else:
            return "just now"  # Less than a minute ago
    except Exception as e:
        print(f"Error @ time_ago_from_string :{e}")
        return ''


def format_datetime(input_date_string):
    input_format = "%Y-%m-%d %H:%M:%S"
    dt_object = datetime.strptime(str(input_date_string), input_format)
    return dt_object.strftime("%Y-%m-%dT%H:%M:%SZ")



import json
from datetime import datetime

import requests

from common.secretmanager import SecretManager
from db.databases import RDSConnection
from security.constants import HONEYPOT_FINDER_URL, GO_PLUS_URL, INSERT_OR_UPDATE_ETH_SECURITY_DATA_QUERY, \
    UPDATE_OTHER_CHAIN_SECURITY_DATA_QUERY, ID_TO_CHAIN_MAP, GET_QUICK_AUDIT_URL, QUICK_INTEL_API_KEY_SECRET_ID, \
    UPDATE_OTHER_CHAIN_SECURITY_DATA_FROM_QUICK_INTEL_QUERY, SOCIAL_INTELLIGENCE_DB_SECRET_ID
from common_utils.chain_utils import chain_to_id, id_to_chain

write_db = RDSConnection(db_secret_name=SOCIAL_INTELLIGENCE_DB_SECRET_ID)


class Honeypot:
    def __init__(self):
        self.__aws_secret_manager = SecretManager()
        self.__api_key_values = self.__aws_secret_manager.get_secret_key_value(
            secret_name=QUICK_INTEL_API_KEY_SECRET_ID)
        self.__api_key = self.__api_key_values.get("api-key")

    @classmethod
    def __store_security_data(cls, security_data, chain_id, from_qi=False):
        if security_data:
            token_id = security_data.get("token_id")
            honeypot = security_data.get("honeypot")
            buy_tax = security_data.get("buy_tax")
            sell_tax = security_data.get("sell_tax")
            holder_count = security_data.get("holder_count")
            lp_burned = security_data.get("lp_burned")
            is_scam = security_data.get("is_scam")
            can_burn = security_data.get("can_burn")
            can_mint = security_data.get("can_mint")
            can_freeze = security_data.get("can_freeze")

            if chain_id == 1:
                value = (token_id, honeypot, sell_tax, buy_tax)
                query = INSERT_OR_UPDATE_ETH_SECURITY_DATA_QUERY
            else:
                if from_qi:
                    value = (honeypot, sell_tax, buy_tax, lp_burned, is_scam, can_burn, can_mint, can_freeze,
                             datetime.utcnow(), token_id)
                    query = UPDATE_OTHER_CHAIN_SECURITY_DATA_FROM_QUICK_INTEL_QUERY
                else:
                    value = (honeypot, sell_tax, buy_tax, holder_count, datetime.utcnow(), token_id)
                    query = UPDATE_OTHER_CHAIN_SECURITY_DATA_QUERY

            try:
                write_db.execute_query(query=query, values=value)
                print(f"INFO: Data updated for: {token_id}")
            except Exception as error:
                print(f"ERROR: {error}")

    def get_security_data(self, chain_id, token_id):
        if not isinstance(chain_id, int):
            chain_id = chain_to_id.get(chain_id)

        if chain_id == 1:
            security_data_dict = self.get_data_from_honeypot(chain_id=chain_id, token_id=token_id)
            if security_data_dict.get("buy_tax") is None or security_data_dict.get("sell_tax") is None:
                return self.security_data_dict_for_non_eth(chain_id, token_id)
            else:
                return security_data_dict
        else:
            return self.security_data_dict_for_non_eth(chain_id, token_id)

    def security_data_dict_for_non_eth(self, chain_id, token_id):
        security_data_dict = self.get_data_from_goplus(chain_id, token_id)
        if security_data_dict.get("buy_tax") is None or security_data_dict.get("sell_tax") is None:
            security_data_dict = self.get_data_from_quickintel(chain_id, token_id)
            return security_data_dict
        return security_data_dict

    @classmethod
    def get_data_from_goplus(cls, chain_id, token_id):
        if isinstance(chain_id, str):
            chain_id = chain_to_id.get(chain_id)

        token_id = token_id.lower()
        url = GO_PLUS_URL.format(chain_id=chain_id, token_id=token_id)
        response = requests.get(url=url)
        try:
            if response.status_code not in [500, 404]:
                result = response.json().get("result")
                if result:
                    is_honey_pot = result.get(token_id).get('is_honeypot')
                    buy_tax = result.get(token_id).get('buy_tax')
                    sell_tax = result.get(token_id).get('sell_tax')
                    holders_count = result.get(token_id).get("holder_count")
                    security_data = {"honeypot": is_honey_pot, "buy_tax": buy_tax, "sell_tax": sell_tax,
                                     "holder_count": holders_count, "token_id": token_id}
                    cls.__store_security_data(security_data=security_data, chain_id=chain_id)
                    return security_data
            else:
                print(f"ERROR: In get_data_from_goplus: {response.text}")
        except Exception as error:
            print(f"ERROR: In get_data_from_goplus {error}")
        return {}

    @classmethod
    def get_data_from_honeypot(cls, chain_id, token_id):
        if isinstance(chain_id, int):
            chain_id = id_to_chain.get(chain_id)

        url = HONEYPOT_FINDER_URL.format(token_id=token_id, chain_id=chain_id)
        response = requests.get(url=url)
        try:
            if response.status_code not in [500, 404]:
                data = response.json()
                is_honey_pot = data.get('IsHoneypot')
                buy_tax = round(data.get('BuyTax'))
                sell_tax = round(data.get('SellTax'))
                security_data = {"honeypot": is_honey_pot, "buy_tax": buy_tax, "sell_tax": sell_tax,
                                 "token_id": token_id}
                cls.__store_security_data(security_data=security_data, chain_id=chain_id)
                return security_data
            else:
                print(f"ERROR: In get_data_from_goplus: {response.text}")
        except Exception as error:
            print(f"ERROR: In get_data_from_honeypot {error}")
        return {}

    def get_data_from_quickintel(self, chain_id, token_id):
        if isinstance(chain_id, int):
            chain_id = id_to_chain.get(chain_id)

        url = GET_QUICK_AUDIT_URL
        payload = {"chain": chain_id, "tokenAddress": token_id}
        headers = {"X-QKNTL-KEY": self.__api_key, "Content-Type": "application/json"}
        response = requests.post(url=url, data=json.dumps(payload), headers=headers)
        try:
            if response.status_code not in [500, 404]:
                data = response.json()
                token_dynamic_details = data.get("tokenDynamicDetails") if data.get("tokenDynamicDetails") else {}
                quick_audit = data.get("quickiAudit") if data.get("quickiAudit") else {}
                honeypot = token_dynamic_details.get("is_Honeypot")
                buy_tax = token_dynamic_details.get("buy_Tax")
                sell_tax = token_dynamic_details.get("sell_Tax")
                lp_burned = token_dynamic_details.get("lp_Burned_Percent")
                is_scam = data.get("isScam")
                can_burn = quick_audit.get("can_Burn")
                can_mint = quick_audit.get("can_Mint")
                can_freeze = quick_audit.get("can_Freeze_Trading")
                security_data = {"honeypot": honeypot, "buy_tax": buy_tax, "sell_tax": sell_tax, "lp_burned": lp_burned,
                                 "is_scam": is_scam, "can_burn": can_burn, "can_mint": can_mint,
                                 "can_freeze": can_freeze, "token_id": token_id}
                self.__store_security_data(security_data=security_data, chain_id=chain_id, from_qi=True)
                return security_data
            else:
                print(f"ERROR: In get_data_from_goplus: {response.text}")
        except Exception as error:
            print(f"ERROR: In get_data_from_quickintel {error}")
        return {}



def extract_twitter_username(twitter_link):
    """
    Extracts the Twitter username from a given input, which can be a URL or just the username itself.

    Args:
        twitter_link (str): The input containing a Twitter URL or just the username.

    Returns:
        str or None: The extracted Twitter username if found, otherwise None.
    """

    start_index = None
    is_valid_url = False
    is_url = False

    if twitter_link.startswith("https://") or twitter_link.startswith("http://"):
        is_url = True
    # Strip off "https://" or "http://" if present
    twitter_link = twitter_link.replace("https://", "").replace("http://", "")

    if "twitter.com" in twitter_link:
        # Find the start index of the username
        is_valid_url = False
        str_find = twitter_link.find("twitter.com/")
        if str_find != -1:
            start_index = str_find + len("twitter.com/")

    elif "x.com" in twitter_link:
        # Find the start index of the username
        is_valid_url = False
        str_find = twitter_link.find("x.com/")
        if str_find != -1:
            start_index = str_find + len("x.com/")

    if start_index :
        # Extract the substring containing the username
        username = twitter_link[start_index:].split("/")[0].split("?")[0]
    else:
        if not is_valid_url and is_url:
            return None
        else:
            username = twitter_link.split("/")[0].split("?")[0]

    return username if username else None


def extract_telegram_channel_name(telegram_link):
    """
    Extracts the Telegram username from a given input, which can be a URL or just the username itself.

    Args:
        telegram_link (str): The input containing a telegram URL or just the username.

    Returns:
        str or None: The extracted telegram username if found, otherwise None.
    """

    start_index = None
    valid_url = False
    is_url = False
    if telegram_link.startswith("https://") or telegram_link.startswith("http://"):
        is_url = True

    # Strip off "https://" or "http://" if present
    telegram_link = telegram_link.replace("https://", "").replace("http://", "")

    if "t.me" in telegram_link:
        valid_url = True
        # Find the start index of the username
        str_find = telegram_link.find("t.me/")
        if str_find != -1:
            start_index = str_find + len("t.me/")
    elif "telegram.me" in telegram_link:
        valid_url = True
        # Find the start index of the username
        str_find = telegram_link.find("telegram.me/")
        if str_find != -1:
            start_index = str_find + len("telegram.me/")

    if start_index:
        # Extract the substring containing the username
        username = telegram_link[start_index:].split("/")[0].split("?")[0]
    else:
        if not valid_url and is_url:
            return None
        else:
            username = telegram_link.split("/")[0].split("?")[0]

    return username if username else None



SOCIAL_INTELLIGENCE_DB_SECRET_ID = "social-intelligence-db"
QUICK_INTEL_API_KEY_SECRET_ID = "quick-intel-api-key"

HONEYPOT_FINDER_URL = "https://api.honeypot.is/legacy/aws/isHoneypot?chain={chain_id}&token={token_id}"
GO_PLUS_URL = "https://api.gopluslabs.io/api/v1/token_security/{chain_id}?contract_addresses={token_id}"
GET_QUICK_AUDIT_URL = "https://api.quickintel.io/v1/getquickiauditfull"

INSERT_OR_UPDATE_ETH_SECURITY_DATA_QUERY = """
INSERT INTO social.token_properties (token_id, is_honeypot, sell_tax, buy_tax)
VALUES (%s, %s, %s, %s)
ON DUPLICATE KEY UPDATE
is_honeypot=VALUES(is_honeypot), sell_tax=VALUES(sell_tax), buy_tax=VALUES(buy_tax)
"""

UPDATE_OTHER_CHAIN_SECURITY_DATA_QUERY = """
UPDATE blockchains.tokens
SET is_honeypot = %s, sell_tax = %s, buy_tax = %s, holders_count = %s, security_updated_at = %s
WHERE address = %s
"""

UPDATE_OTHER_CHAIN_SECURITY_DATA_FROM_QUICK_INTEL_QUERY = """
UPDATE blockchains.tokens
SET is_honeypot = %s, sell_tax = %s, buy_tax = %s,
lp_burned = %s, is_scam = %s, can_burn = %s, can_mint = %s,
can_freeze = %s, security_updated_at = %s
WHERE address = %s
"""

CHAIN_TO_ID_MAP = {
    "ethereum": 1,
    "optimism": 10,
    "cronos": 25,
    "bsc": 56,
    "okex-chain": 66,
    "okt chain": 66,
    "gnosis": 100,
    "heco": 128,
    "polygon": 137,
    "fantom": 250,
    "kucoin": 321,
    "kcc": 321,
    "kucoin token": 321,
    "zksync": 324,
    "ethw": 10001,
    "fon": 201022,
    "arbitrum": 42161,
    "arbitrum-nova": 42161,
    "arbitrum-one": 42161,
    "avalanche": 43114,
    "linea": 59144,
    "base": 8453,
    "tron": "tron",
    "scroll": 534352,
    "solana": "solana",
    "eth": "eth",
    "core": "core",
    "polygonzkevm": "polygonzkevm",
    "loop": "loop",
    "kava": "kava",
    "metis": "metis",
    "astar": "astar",
    "oasis": "oasis",
    "iotex": "iotex",
    "conflux": "conflux",
    "canto": "canto",
    "energi": "energi",
    "velas": "velas",
    "grove": "grove",
    "pulse": "pulse",
    "besc": "besc",
    "shibarium": "shibarium",
    "opbnb": "opbnb",
    "bitrock": "bitrock",
    "mantle": "mantle",
    "lightlink": "lightlink",
    "klaytn": "klaytn",
    "injective": "injective",
    "radix": "radix",
    "sui": "sui",
    "manta": "manta",
    "zeta": "zeta"
}

ID_TO_CHAIN_MAP = {
    1: "eth",
    10: "optimism",
    25: "cronos",
    56: "bsc",
    100: "gnosis",
    128: "heco",
    137: "polygon",
    250: "fantom",
    324: "zksync",
    10001: "ethw",
    201022: "fon",
    42161: "arbitrum",
    43114: "avalanche",
    59144: "linea",
    8453: "base",
    534352: "scroll",
    "solana": "solana",
    "eth": "eth",
    "arbitrum": "arbitrum",
    "bac": "bsc",
    "polygon": "polygon",
    "fantom": "fantom",
    "avalanche": "avalanche",
    "core": "core",
    "zksync": "zksync",
    "polygonzkevm": "polygonzkevm",
    "loop": "loop",
    "kava": "kava",
    "metis": "metis",
    "astar": "astar",
    "oasis": "oasis",
    "iotex": "iotex",
    "conflux": "conflux",
    "canto": "canto",
    "energi": "energi",
    "velas": "velas",
    "grove": "grove",
    "pulse": "pulse",
    "besc": "besc",
    "linea": "linea",
    "base": "base",
    "shibarium": "shibarium",
    "opbnb": "opbnb",
    "bitrock": "bitrock",
    "optimism": "optimism",
    "mantle": "mantle",
    "lightlink": "lightlink",
    "klaytn": "klaytn",
    "injective": "injective",
    "radix": "radix",
    "sui": "sui",
    "manta": "manta",
    "zeta": "zeta",
}



import json

import boto3
from botocore.exceptions import ClientError


class SecretManager:

    def __init__(self, access_key=None, secret_key=None, region_name=None):
        self.__access_key = access_key
        self.__secret_key = secret_key
        self.__region_name = region_name

    def get_secrets_manager_client(self):
        print('INFO: In secrets_manager_client')
        try:
            session = boto3.session.Session(aws_access_key_id=self.__access_key,
                                            aws_secret_access_key=self.__secret_key)
            return session.client(service_name="secretsmanager", region_name=self.__region_name)
        except Exception as e:
            print(f'Exception while creating secrets_manager_client: {e}')
            raise e

    # Retrieves the secret value for the given secret name from AWS Secrets Manager.
    def get_secret_key_value(self, secret_name):
        print(f"INFO: In get_secret_key_value for: {secret_name}")
        client = self.get_secrets_manager_client()
        try:
            get_secret_key_value_response = client.get_secret_value(SecretId=secret_name)
        except ClientError as e:
            print(f"An error occurred while retrieving the secret for {secret_name}: {e}")
            raise e

        return json.loads(get_secret_key_value_response['SecretString'])



from common.secretmanager import SecretManager

EXPERIMENTS_SECRET_NAME = "experiments"
ENABLED_TREATMENT = "enabled"
CONTROL_TREATMENT = "control"


class Experiment:

    def __init__(self, access_key, secret_key, region_name):
        self.access_key = access_key
        self.secret_key = secret_key
        self.region_name = region_name

    def get_experiment(self, experiment):
        aws_secret_manager = SecretManager(self.access_key, self.secret_key, self.region_name)
        experiment_values = aws_secret_manager.get_secret_key_value(EXPERIMENTS_SECRET_NAME)
        return experiment_values.get(experiment)

    def is_enabled(self, experiment):
        return ENABLED_TREATMENT == self.get_experiment(experiment)

    def is_control(self, experiment):
        return CONTROL_TREATMENT == self.get_experiment(experiment)
